var character;
var canvas = document.getElementById("cvGame");
var ctx = canvas.getContext("2d");
var game = {
    states: Object.freeze({"start": 0, "running":1, "over": 2,"end" : 3}),
    speed: 50,
    state: null,
    tmr: null,
    gravity: 0.75,
    jump: -12,
    barDown: false
};
var Sprig = function(){
    this.x = 0;
    this.y = 0;
    this.moveX = 0;
    this.moveY = 0;
    this.visible = false;
};
var Sprite = function(imgSrc){
    Sprig.call(this);
    this.img = new Image();
    this.img.src = imgSrc;
    this.continuous = false;
};
Sprite.prototype.getRect = function() {
    return {x: this.x, y: this.y, width: this.img.width, height: this.img.height};
};
var Anim = function(imgSrc, frames, frameWidth, repeat){
    this.img = new Image();
    this.img.src = imgSrc;
    this.frames = frames;
    this.frameWidth = frameWidth;
    this.frame = -1;
    this.default = 0;
    this.repeat = repeat;
    this.reset = function() {this.frame = -1;};
};
Anim.prototype.animate = function(x, y) {
    if(game.state == game.states.running){
        if(this.repeat){
            this.frame++;
            this.frame %= this.frames;
        } else {
            if(this.frame < this.frames -1){
                this.frame++;
            }
        }
    } else {
        this.frame = this.default; // stop animation
    }
    let clipx = this.frame * this.frameWidth;
    ctx.drawImage(this.img, clipx, 0, this.frameWidth, this.img.height, x, y, this.frameWidth, this.img.height);
};
// Anim.prototype.getSize = function() {
//     return {width: this.frameWidth, height: this.img.height};
// };
Anim.prototype.getRect = function(x, y) {
    return {x: x, y: y, width: this.frameWidth, height: this.img.height};
};
var Actor = function(){
    Sprig.call(this);
    this.anims = [];
    this.current = 0;
    this.height = 0;
    this.manageActions = function(){};
    this.resetCurrent = function(current) {
        this.current = current;
        this.anims[this.current].reset();
    };
};
// Actor.prototype.getRect = function() {
//     let dims = this.anims[this.current].getSize();
//     dims.x = this.x;
//     dims.y = this.y;
//     return dims;
// };
Actor.prototype.getRect = function() {
    return this.anims[this.current].getRect(this.x, this.y);
};
Sprite.prototype.animate = function() {
    if(this.visible){
        ctx.drawImage(this.img, this.x, this.y);
    }
    if(game.state === game.states.running || game.state === game.states.over){
        this.x += this.moveX;
        this.y += this.moveY;
        if(this.continuous && this.x < (canvas.width - this.img.width)){
            this.x = 0;
        }
    }
};
Actor.prototype.animate = function() {
    if(this.visible){
        this.anims[this.current].animate(this.x, this.y);
    }
    if(game.state == game.states.running){
        this.x += this.moveX;
        this.y += this.moveY;
    }
    this.manageActions();
};
// demo of pragmatic approach but risk e.g. introducing scale
var Rect = function(sprite){
    this.x = sprite.x;
    this.y = sprite.y;
    if(sprite instanceof Actor){
        this.width = sprite.anims[sprite.current].frameWidth;
        this.height = sprite.anims[sprite.current].img.height;
    } else {
        this.width = sprite.img.width;
        this.height = sprite.img.height;
    }
};
function intersect(rectA, rectB){
    // return Math.max(rectA.x, rectB.x) < Math.min(rectA.x + rectA.width, rectB.x + rectB.width) &&
    //     Math.max(rectA.y, rectB.y) < Math.min(rectA.y + rectA.height, rectB.y + rectB.height);
    let rA = rectA.x + rectA.width, rB = rectB.x + rectB.width;//right x
    let bA = rectA.y + rectA.height, bB = rectB.y + rectB.height;//btm y
    let ix = Math.max(rectA.x, rectB.x), iy =  Math.max(rectA.y, rectB.y); //?? min
    let rect = {
        x: ix,
        y: iy,
        width: Math.min(rA, rB) - ix,
        height: Math.min(bA, bB) - iy
    };
    if(rect.x < (rect.x + rect.width) && rect.y < (rect.y + rect.height)){
        return rect;
    }
    return false;
}
function drawFrame() {
    ctx.clearRect(0, 0, canvas.width, canvas.height);
    character.animate();
}
function makeCharacter() {
    character = new Actor();
    character.height = 75;
    character.x = Math.floor(canvas.width / 5);
    character.y = canvas.height - character.height;
    let a = new Anim("images/walk11x60x75.png", 11, 60, true);
    character.anims.push(a);
    a = new Anim("images/jumpup6x60x75.png", 6, 60, false);
    character.anims.push(a);
    a = new Anim("images/jumpdown6x65x75.png", 6, 65, false);
    character.anims.push(a);
    character.visible = true;
    character.hadBoost = false;
    addCharacterActions();
}
function addCharacterActions() {
    character.jump = function() {
        if(this.current === 1 || this.current === 2) {return;}
        this.hadBoost = false;
        this.resetCurrent(1);
        this.moveY = game.jump;
        this.jumping = true;
    };
    character.manageActions = function() {
        if(this.current === 1){
            this.moveY += game.gravity;
            if(this.moveY >= 0){
                this.resetCurrent(2);
            }
        } else if(this.current === 2){
            this.moveY += game.gravity; // TV ??
            if(this.y >= canvas.height - this.height){
                this.moveY = 0;
                this.y = canvas.height - this.height;
                this.resetCurrent(0);
            }
        }
    };
}
function keyPressed(e) {
    switch (game.state){
       case game.states.start:
            // manage start of game 
           break;
       case game.states.running:
           //manage character actions
           if(e.keyCode === 32){
               game.barDown = true;
               character.jump();
           }
           if(e.keyCode === 37){
               // testing
               let r1 = new Rect(character);
               let r2 = character.getRect();
               if(r1.x === r2.x && r1.y === r2.y && r1.width === r2.width && r1.height === r2.height){
                   alert("Rects match");
               } else {
                   alert("Rects not equal");
               }
           }
           break;
       case game.states.end:
           // manage restart
           break;
   }
}
function keyReleased(e) {
    if(e.keyCode == 32){
        // space bar released
        game.barDown = false;
    }
}
function initialise() {
    canvas.width = 800;
    canvas.height = 500;
    window.addEventListener("keydown", keyPressed);
    window.addEventListener("keyup", keyReleased);
    makeCharacter();
    game.state = game.states.running; // testing
    game.tmr = setInterval(drawFrame, game.speed);
}
initialise();